@@ -21,8 +21,9 @@
 
 #include "documentmanager.h"
 
-#include "toolmanager.h"
 #include "abstracttool.h"
+#include "maprenderer.h"
+#include "toolmanager.h"
 
 #include <QTabWidget>
 #include <QUndoGroup>
@@ -31,6 +32,21 @@
 using namespace Tiled;
 using namespace Tiled::Internal;
 
+DocumentManager *DocumentManager::mInstance = 0;
+
+DocumentManager *DocumentManager::instance()
+{
+    if (!mInstance)
+        mInstance = new DocumentManager;
+    return mInstance;
+}
+
+void DocumentManager::deleteInstance()
+{
+    delete mInstance;
+    mInstance = 0;
+}
+
 DocumentManager::DocumentManager(QObject *parent)
     : QObject(parent)
     , mTabWidget(new QTabWidget)
@@ -139,7 +155,7 @@ void DocumentManager::addDocument(MapDocument *mapDocument)
 
     scene->setMapDocument(mapDocument);
     view->setScene(scene);
-    view->centerOn(0, 0);
+    view->centerOn(mapDocument->renderer()->tileToPixelCoords(0, 0));
 
     const int documentIndex = mDocuments.size() - 1;
 
